@@ -33,7 +33,6 @@ gi.require_version('Gst', '1.0')
 gi.require_version('GstAudio', '1.0')
 gi.require_version('GstPbutils', '1.0')
 from gi.repository import Gst, GstAudio, GstPbutils, GObject, Gtk, Gdk, Pango, GdkPixbuf, Gio, GLib
-from .gi_composites import GtkTemplate
 
 if Gtk.get_major_version() < 3 or Gtk.get_minor_version() < 14:
     sys.exit('Gtk 3.14 is required')
@@ -44,7 +43,7 @@ from .gobject_worker import GObjectWorker
 from .pandora import *
 from .pandora.data import *
 from .plugin import load_plugins
-from .util import parse_proxy, open_browser, SecretService, popup_at_pointer
+from .util import parse_proxy, open_browser, SecretService, popup_at_pointer, is_flatpak
 from .migrate_settings import maybe_migrate_settings
 
 try:
@@ -177,7 +176,7 @@ class CellRendererAlbumArt(Gtk.CellRenderer):
         bg_rgb = bg_color.to_string()
 
         # Use our color values to create strings representing valid SVG's
-        # for backgound and rate_bg, then load them with PixbufLoader.
+        # for background and rate_bg, then load them with PixbufLoader.
         background = BACKGROUND_SVG.format(px=ALBUM_ART_SIZE, fg=fg_rgb).encode()
         rating_bg = RATING_BG_SVG.format(bg=bg_rgb).encode()
 
@@ -212,7 +211,7 @@ class CellRendererAlbumArt(Gtk.CellRenderer):
         icon, was_symbolic = info.load_symbolic(fg_color, fg_color, fg_color, fg_color)
         self.tired_icon = icon.scale_simple(12, 12, GdkPixbuf.InterpType.BILINEAR)
 
-@GtkTemplate(ui='/io/github/Pithos/ui/PithosWindow.ui')
+@Gtk.Template(resource_path='/io/github/Pithos/ui/PithosWindow.ui')
 class PithosWindow(Gtk.ApplicationWindow):
     __gtype_name__ = "PithosWindow"
     __gsignals__ = {
@@ -230,24 +229,24 @@ class PithosWindow(Gtk.ApplicationWindow):
         "player-ready": (GObject.SignalFlags.RUN_FIRST, None, (GObject.TYPE_BOOLEAN,)),
     }
 
-    volume = GtkTemplate.Child()
-    playpause_image = GtkTemplate.Child()
-    statusbar = GtkTemplate.Child()
-    song_menu = GtkTemplate.Child()
-    song_menu_love = GtkTemplate.Child()
-    song_menu_unlove = GtkTemplate.Child()
-    song_menu_ban = GtkTemplate.Child()
-    song_menu_unban = GtkTemplate.Child()
-    song_menu_create_station = GtkTemplate.Child()
-    song_menu_create_song_station = GtkTemplate.Child()
-    song_menu_create_artist_station = GtkTemplate.Child()
-    songs_treeview = GtkTemplate.Child()
-    stations_button = GtkTemplate.Child()
-    stations_label = GtkTemplate.Child()
+    volume = Gtk.Template.Child()
+    playpause_image = Gtk.Template.Child()
+    statusbar = Gtk.Template.Child()
+    song_menu = Gtk.Template.Child()
+    song_menu_love = Gtk.Template.Child()
+    song_menu_unlove = Gtk.Template.Child()
+    song_menu_ban = Gtk.Template.Child()
+    song_menu_unban = Gtk.Template.Child()
+    song_menu_create_station = Gtk.Template.Child()
+    song_menu_create_song_station = Gtk.Template.Child()
+    song_menu_create_artist_station = Gtk.Template.Child()
+    songs_treeview = Gtk.Template.Child()
+    stations_button = Gtk.Template.Child()
+    stations_label = Gtk.Template.Child()
 
-    api_update_dialog_real = GtkTemplate.Child()
-    error_dialog_real = GtkTemplate.Child()
-    fatal_error_dialog_real = GtkTemplate.Child()
+    api_update_dialog_real = Gtk.Template.Child()
+    error_dialog_real = Gtk.Template.Child()
+    fatal_error_dialog_real = Gtk.Template.Child()
 
     def __init__(self, app, test_mode):
         super().__init__(application=app)
@@ -366,12 +365,15 @@ class PithosWindow(Gtk.ApplicationWindow):
         self.worker = GObjectWorker()
 
         try:
-            self.tempdir = tempfile.TemporaryDirectory(prefix='pithos-',
-                                                       dir=GLib.get_user_cache_dir())
-            logging.info("Created temporary directory %s" %self.tempdir.name)
+            tempdir_base = '/var/tmp' # Preferred over /tmp as lots of icons can be large in size.
+            if is_flatpak():
+                # However in flatpak that path is not readable by the host.
+                tempdir_base = os.path.join(GLib.get_user_cache_dir(), 'tmp')
+            self.tempdir = tempfile.TemporaryDirectory(prefix='pithos-', dir=tempdir_base)
+            logging.info("Created temporary directory {}".format(self.tempdir.name))
         except IOError as e:
             self.tempdir = None
-            logging.warning('Failed to create a temporary directory')
+            logging.warning('Failed to create a temporary directory: {}'.format(e))
 
     @property
     def playing(self):
@@ -413,6 +415,7 @@ class PithosWindow(Gtk.ApplicationWindow):
         self.stations_popover.set_model(self.stations_model)
         self.stations_popover.listbox.connect('row-activated', self.active_station_changed)
         self.stations_button.set_popover(self.stations_popover)
+        self.stations_popover.search.connect('activate', self.search_activate_handler)
 
     def init_actions(self, app):
         action = Gio.SimpleAction.new('playpause', None)
@@ -470,6 +473,11 @@ class PithosWindow(Gtk.ApplicationWindow):
         app.add_accelerator('<Primary>d', 'win.bookmark', None)
         action.connect('activate', self.bookmark_song)
 
+        action = Gio.SimpleAction.new('toggle-station-popover', None)
+        self.add_action(action)
+        app.add_accelerator('<Primary>r', 'win.toggle-station-popover', None)
+        action.connect('activate', self.stations_popover.toggle_visibility)
+
     def worker_run(self, fn, args=(), callback=None, message=None, context='net', errorback=None, user_data=None):
         if context and message:
             self.statusbar.push(self.statusbar.get_context_id(context), message)
@@ -760,7 +768,6 @@ class PithosWindow(Gtk.ApplicationWindow):
         self.emit('song-changed', song)
         self.emit('metadata-changed', song)
 
-    @GtkTemplate.Callback
     def next_song(self, *ignore):
         if self.current_song_index is not None:
             self.start_song(self.current_song_index + 1)
@@ -825,7 +832,6 @@ class PithosWindow(Gtk.ApplicationWindow):
             # desired state is playing when going to a new song.
             self.playpause_image.set_from_icon_name('media-playback-pause-symbolic', Gtk.IconSize.SMALL_TOOLBAR)
 
-    @GtkTemplate.Callback
     def user_playpause(self, *ignore):
         self.playpause_notify()
 
@@ -873,7 +879,7 @@ class PithosWindow(Gtk.ApplicationWindow):
             file_url = None
             if tmpdir:
                 try:
-                    with tempfile.NamedTemporaryFile(prefix='art-', dir=tmpdir.name, delete=False) as f:
+                    with tempfile.NamedTemporaryFile(prefix='art-', suffix='.jpeg', dir=tmpdir.name, delete=False) as f:
                         f.write(image)
                         file_url = urllib.parse.urljoin('file://', urllib.parse.quote(f.name))
                 except IOError:
@@ -1062,7 +1068,7 @@ class PithosWindow(Gtk.ApplicationWindow):
             return True
 
     def on_gst_stream_start(self, bus, message):
-        # Edge case. We might get this singal while we're reconnecting to Pandora.
+        # Edge case. We might get this signal while we're reconnecting to Pandora.
         # If so self.current_song will be None.
         if self.current_song is None:
             return
@@ -1070,7 +1076,6 @@ class PithosWindow(Gtk.ApplicationWindow):
         self.current_song.duration = self.query_duration() or self.current_song.trackLength * Gst.SECOND
         self.current_song.duration_message = self.format_time(self.current_song.duration)
         self.update_song_row()
-        self.check_if_song_is_ad()
         # We can't seek so duration in MPRIS is just for display purposes if it's not off by more than a sec it's OK.
         if self.current_song.get_duration_sec() != self.current_song.trackLength:
             self.emit('metadata-changed', self.current_song)
@@ -1109,36 +1114,22 @@ class PithosWindow(Gtk.ApplicationWindow):
         if not GstPbutils.install_plugins_installation_in_progress():
             self.next_song()
 
-    def check_if_song_is_ad(self):
-        if self.current_song.is_ad is None:
-            if self.current_song.duration:
-                if self.current_song.get_duration_sec() < 45:  # Less than 45 seconds we assume it's an ad
-                    logging.info('Ad detected!')
-                    self.current_song.is_ad = True
-                    self.update_song_row()
-                    self.set_title("Commercial Advertisement - Pithos")
-                else:
-                    logging.info('Not an Ad..')
-                    self.current_song.is_ad = False
-            else:
-                logging.warning('dur_stat is False. The assumption that duration is available once the stream-start messages feeds is bad.')
-
     def on_gst_buffering(self, bus, message):
         # React to the buffer message immediately and also fire a short repeating timeout
         # to check the buffering state that cancels only if we're not buffering or there's a pending timeout.
         # This will insure we don't get stuck in a buffering state if we're really not buffering.
 
-        self.react_to_buffering_mesage(False)
+        self.react_to_buffering_message(False)
 
         if self.buffering_timer_id:
             GLib.source_remove(self.buffering_timer_id)
             self.buffering_timer_id = 0
-        self.buffering_timer_id = GLib.timeout_add(200, self.react_to_buffering_mesage, True)
+        self.buffering_timer_id = GLib.timeout_add(200, self.react_to_buffering_message, True)
 
-    def react_to_buffering_mesage(self, from_timeout):
+    def react_to_buffering_message(self, from_timeout):
         # If the pipeline signals that it is buffering set the player to PseudoGst.BUFFERING
         # (which is an alias to Gst.State.PAUSED). During buffering if the user goes to Pause
-        # or Play(an/or back again) go though all the motions but don't actaully change the
+        # or Play(an/or back again) go though all the motions but don't actually change the
         # player's state to the desired state until buffering has completed. The player only
         # cares about the actual state, the rest of Pithos only cares about our buffer_recovery
         # state, the state we *wish* we were in.
@@ -1247,6 +1238,13 @@ class PithosWindow(Gtk.ApplicationWindow):
             GLib.source_remove(self.ui_loop_timer_id)
             self.ui_loop_timer_id = 0
 
+    def search_activate_handler(self,station):
+        row = self.stations_popover.listbox.get_row_at_y(0)
+        if not row:
+            return
+        self.station_changed(row.station)
+        self.stations_popover.on_row_activated(self.stations_popover.listbox, row)
+
     def active_station_changed(self, listbox, row):
         self.station_changed(row.station)
 
@@ -1323,35 +1321,35 @@ class PithosWindow(Gtk.ApplicationWindow):
         song = song or self.current_song
         open_browser(song.songDetailURL)
 
-    @GtkTemplate.Callback
+    @Gtk.Template.Callback()
     def on_menuitem_love(self, widget):
         self.love_song(song=self.selected_song())
 
-    @GtkTemplate.Callback
+    @Gtk.Template.Callback()
     def on_menuitem_ban(self, widget):
         self.ban_song(song=self.selected_song())
 
-    @GtkTemplate.Callback
+    @Gtk.Template.Callback()
     def on_menuitem_unrate(self, widget):
         self.unrate_song(song=self.selected_song())
 
-    @GtkTemplate.Callback
+    @Gtk.Template.Callback()
     def on_menuitem_tired(self, widget):
         self.tired_song(song=self.selected_song())
 
-    @GtkTemplate.Callback
+    @Gtk.Template.Callback()
     def on_menuitem_info(self, widget):
         self.info_song(song=self.selected_song())
 
-    @GtkTemplate.Callback
+    @Gtk.Template.Callback()
     def on_menuitem_bookmark_song(self, widget):
         self.bookmark_song(song=self.selected_song())
 
-    @GtkTemplate.Callback
+    @Gtk.Template.Callback()
     def on_menuitem_bookmark_artist(self, widget):
         self.bookmark_song_artist(self.selected_song())
 
-    @GtkTemplate.Callback
+    @Gtk.Template.Callback()
     def on_menuitem_create_artist_station(self, widget):
         user_date = 'artist', html.escape(self.selected_song().artist)
         self.worker_run(
@@ -1361,7 +1359,7 @@ class PithosWindow(Gtk.ApplicationWindow):
             user_data=user_date,
         )
 
-    @GtkTemplate.Callback
+    @Gtk.Template.Callback()
     def on_menuitem_create_song_station(self, widget):
         title = html.escape(self.selected_song().title)
         artist = html.escape(self.selected_song().artist)
@@ -1414,7 +1412,7 @@ class PithosWindow(Gtk.ApplicationWindow):
     def volume_down(self, *ignore):
         self.adjust_volume(-2)
 
-    @GtkTemplate.Callback
+    @Gtk.Template.Callback()
     def on_volume_change_event(self, volumebutton, value):
         self.set_player_volume(value)
 
@@ -1478,7 +1476,7 @@ class PithosWindow(Gtk.ApplicationWindow):
         self.restore_position()
         Gtk.Window.present(self)
 
-    @GtkTemplate.Callback
+    @Gtk.Template.Callback()
     def on_configure_event(self, *ignore):
         # Getting and setting window position does not work in Wayland.
         if self.not_in_x:
@@ -1490,7 +1488,7 @@ class PithosWindow(Gtk.ApplicationWindow):
         """quit - signal handler for closing the PithosWindow"""
         Gio.Application.get_default().quit()
 
-    @GtkTemplate.Callback
+    @Gtk.Template.Callback()
     def on_destroy(self, widget, data=None):
         """on_destroy - called when the PithosWindow is close. """
         self.stop()
